package cn.com.qmth.examcloud.core.questions.api.controller;

import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.questions.api.request.GetDefaultQuesionReq;
import cn.com.qmth.examcloud.core.questions.api.request.GetDefaultQuesionsReq;
import cn.com.qmth.examcloud.core.questions.api.request.GetQuestionListReq;
import cn.com.qmth.examcloud.core.questions.api.request.GetQuestionReq;
import cn.com.qmth.examcloud.core.questions.dao.entity.ExtractConfig;
import cn.com.qmth.examcloud.core.questions.service.ExtractConfigProviderService;
import cn.com.qmth.examcloud.core.questions.service.ExtractConfigService;
import cn.com.qmth.examcloud.core.questions.service.QuestionProviderService;
import cn.com.qmth.examcloud.question.commons.core.question.DefaultQuestion;
import cn.com.qmth.examcloud.question.commons.core.question.DefaultQuestionStructure;
import cn.com.qmth.examcloud.question.commons.core.question.DefaultQuestionUnit;
import cn.com.qmth.examcloud.support.handler.QuestionBodyHandler;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Set;

/**
 * @author weiwenhai
 * @date 20180.9.10
 * @company qmth
 * @description controller
 * @code 013
 */
@RestController
@RequestMapping("${api_cqb}/default_question")
public class DefaultQuesionController extends ControllerSupport {

    private static final Logger log = LoggerFactory.getLogger(DefaultQuesionController.class);

    @Autowired
    private QuestionProviderService questionProviderService;

    @Autowired
    private ExtractConfigProviderService extractConfigExamService;

    @Autowired
    private ExtractConfigService extractConfigService;

    @ApiOperation(value = "外部接口保存单个试题", notes = "外部接口保存单个试题")
    @PostMapping("/save")
    public ResponseEntity<Object> save(@RequestBody GetDefaultQuesionReq req) {
        User user = getAccessUser();
        if (user == null) {
            throw new StatusException("Q-013041", "用户不存在！");
        }
        DefaultQuestion defaultQuestion = req.getDefaultQuestion();
        if (defaultQuestion.getRootOrgId() == null) {
            throw new StatusException("Q-013045", "rootOrgId is null");
        }
        String id = questionProviderService.save(defaultQuestion);
        return new ResponseEntity<>(id, HttpStatus.OK);
    }

    @ApiOperation(value = "修改试题")
    @PutMapping("/update")
    public ResponseEntity<Object> update(@RequestBody GetDefaultQuesionReq req) {
        User user = getAccessUser();
        if (user == null) {
            throw new StatusException("Q-013057", "用户不存在！");
        }
        DefaultQuestion defaultQuestion = req.getDefaultQuestion();
        if (defaultQuestion.getRootOrgId() == null) {
            throw new StatusException("Q-013061", "rootOrgId is null");
        }
        String id = questionProviderService.save(defaultQuestion);
        return new ResponseEntity<>(id, HttpStatus.OK);
    }

    @ApiOperation(value = "查询试题")
    @PostMapping("/find")
    public ResponseEntity<Object> find(@RequestBody GetDefaultQuesionsReq req) {
        User user = getAccessUser();
        if (user == null) {
            throw new StatusException("Q-013075", "用户不存在！");
        }
        Long rootOrgId = req.getRootOrgId();
        if (rootOrgId == null) {
            throw new StatusException("Q-011079", "rootOrgId is null");
        }
        Page<DefaultQuestion> defaultQuestions = questionProviderService.findQustions(rootOrgId, req.getProperties(),
                req.getCurPage(), req.getPageSize());
        return new ResponseEntity<>(defaultQuestions, HttpStatus.OK);
    }

    @ApiOperation(value = "查询试题")
    @PostMapping("/find_questions")
    public ResponseEntity<Object> find(@RequestBody GetQuestionListReq req) {
        Set<String> questionIds = req.getQuestionIds();
        if (questionIds == null || questionIds.size() < 1) {
            throw new StatusException("Q-011089", "questionId is null");
        }
        List<DefaultQuestion> defaultQuestions = questionProviderService.findQuestions(questionIds);
        return new ResponseEntity<>(defaultQuestions, HttpStatus.OK);
    }

    @ApiOperation(value = "查询试题详细信息（管理端）")
    @PostMapping("/question/detail")
    public ResponseEntity<Object> findForWebAdmin(@RequestBody GetQuestionReq req) {
        DefaultQuestion defaultQuestion = this.getDefaultQuestion(req, "webAdmin");
        return new ResponseEntity<>(defaultQuestion, HttpStatus.OK);
    }

    @ApiOperation(value = "查询试题详细信息（Web版客户端）")
    @PostMapping("/question")
    public ResponseEntity<Object> findForWebClient(@RequestBody GetQuestionReq req) {
        DefaultQuestion defaultQuestion = this.getDefaultQuestion(req, "webClient");
        this.handeQuestionAnswer(req.getExamId(), req.getCourseCode(), defaultQuestion);

        return new ResponseEntity<>(defaultQuestion, HttpStatus.OK);
    }

    @ApiOperation(value = "查询试题详细信息（C版客户端）")
    @PostMapping("/question/for/client")
    public ResponseEntity<Object> findForPcClient(@RequestBody GetQuestionReq req) {
        DefaultQuestion defaultQuestion = this.getDefaultQuestion(req, "pcClient");
        this.handeQuestionAnswer(req.getExamId(), req.getCourseCode(), defaultQuestion);

        // 将题干、选项等 HTML结构转换为“富文本”JSON结构
        QuestionBodyHandler.convertRichText(defaultQuestion);

        return new ResponseEntity<>(defaultQuestion, HttpStatus.OK);
    }

    private DefaultQuestion getDefaultQuestion(GetQuestionReq req, String fromBy) {
        if (req.getExamId() == null) {
            throw new StatusException("Q-010078", "examId is null");
        }

        if (StringUtils.isBlank(req.getCourseCode())) {
            throw new StatusException("Q-010082", "courseCode is null");
        }

        if (StringUtils.isBlank(req.getGroupCode())) {
            throw new StatusException("Q-010086", "groupCode is null");
        }

        if (StringUtils.isBlank(req.getQuestionId())) {
            throw new StatusException("Q-010090", "questionId is null");
        }

        return extractConfigExamService.getDefaultQuestion(req.getExamId(), req.getCourseCode(), req.getGroupCode(),
                req.getQuestionId(), fromBy);
    }

    private void handeQuestionAnswer(Long examId, String courseCode, DefaultQuestion defaultQuestion) {
        ExtractConfig extractConfig = extractConfigService.findConfig(new ExtractConfig(examId, courseCode));

        boolean hideAnswer = false;
        if (extractConfig == null) {
            // 不存在调卷规则，则业务数据不正常，屏蔽答案
            hideAnswer = true;
            log.warn("$$$异常行为，获取不存在调卷规则的试题敏感数据！questionId:{}", defaultQuestion.getId());
        } else {
            if (ExamType.ONLINE.name().equals(extractConfig.getExamType())
                    || ExamType.ONLINE_HOMEWORK.name().equals(extractConfig.getExamType())) {
                // 目前除了“练习”，“在线考试、在线作业”在学生端使用中不需要显示答案
                hideAnswer = true;
                log.warn("$$$异常行为，获取考试类型为{}的试题敏感数据！questionId:{}", extractConfig.getExamType(), defaultQuestion.getId());
            }
        }

        if (!hideAnswer) {
            return;
        }

        DefaultQuestionStructure structure = defaultQuestion.getMasterVersion();
        if (structure == null) {
            return;
        }
        List<DefaultQuestionUnit> questionUnits = structure.getQuestionUnitList();
        if (CollectionUtils.isEmpty(questionUnits)) {
            return;
        }

        for (DefaultQuestionUnit questionUnit : questionUnits) {
            // 清空答案
            questionUnit.setRightAnswer(null);
        }
    }

}
